"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import dex.v1.pool_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgCreatePool(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CREATOR_FIELD_NUMBER: builtins.int
    POOL_PARAMS_FIELD_NUMBER: builtins.int
    POOL_ASSETS_FIELD_NUMBER: builtins.int
    creator: typing.Text
    @property
    def pool_params(self) -> dex.v1.pool_pb2.PoolParams: ...
    @property
    def pool_assets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[dex.v1.pool_pb2.PoolAsset]: ...
    def __init__(self,
        *,
        creator: typing.Text = ...,
        pool_params: typing.Optional[dex.v1.pool_pb2.PoolParams] = ...,
        pool_assets: typing.Optional[typing.Iterable[dex.v1.pool_pb2.PoolAsset]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool_params",b"pool_params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["creator",b"creator","pool_assets",b"pool_assets","pool_params",b"pool_params"]) -> None: ...
global___MsgCreatePool = MsgCreatePool

class MsgCreatePoolResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_ID_FIELD_NUMBER: builtins.int
    pool_id: builtins.int
    def __init__(self,
        *,
        pool_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id"]) -> None: ...
global___MsgCreatePoolResponse = MsgCreatePoolResponse

class MsgJoinPool(google.protobuf.message.Message):
    """
    Message to join a pool (identified by poolId) with a set of tokens to deposit.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    POOL_ID_FIELD_NUMBER: builtins.int
    TOKENS_IN_FIELD_NUMBER: builtins.int
    sender: typing.Text
    pool_id: builtins.int
    @property
    def tokens_in(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        sender: typing.Text = ...,
        pool_id: builtins.int = ...,
        tokens_in: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id","sender",b"sender","tokens_in",b"tokens_in"]) -> None: ...
global___MsgJoinPool = MsgJoinPool

class MsgJoinPoolResponse(google.protobuf.message.Message):
    """
    Response when a user joins a pool.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POOL_FIELD_NUMBER: builtins.int
    NUM_POOL_SHARES_OUT_FIELD_NUMBER: builtins.int
    REMAINING_COINS_FIELD_NUMBER: builtins.int
    @property
    def pool(self) -> dex.v1.pool_pb2.Pool:
        """the final state of the pool after a join"""
        pass
    @property
    def num_pool_shares_out(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """sum of LP tokens minted from the join"""
        pass
    @property
    def remaining_coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """remaining tokens from attempting to join the pool"""
        pass
    def __init__(self,
        *,
        pool: typing.Optional[dex.v1.pool_pb2.Pool] = ...,
        num_pool_shares_out: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        remaining_coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["num_pool_shares_out",b"num_pool_shares_out","pool",b"pool"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["num_pool_shares_out",b"num_pool_shares_out","pool",b"pool","remaining_coins",b"remaining_coins"]) -> None: ...
global___MsgJoinPoolResponse = MsgJoinPoolResponse

class MsgExitPool(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    POOL_ID_FIELD_NUMBER: builtins.int
    POOL_SHARES_FIELD_NUMBER: builtins.int
    sender: typing.Text
    pool_id: builtins.int
    @property
    def pool_shares(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        sender: typing.Text = ...,
        pool_id: builtins.int = ...,
        pool_shares: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool_shares",b"pool_shares"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id","pool_shares",b"pool_shares","sender",b"sender"]) -> None: ...
global___MsgExitPool = MsgExitPool

class MsgExitPoolResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOKENS_OUT_FIELD_NUMBER: builtins.int
    @property
    def tokens_out(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        tokens_out: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tokens_out",b"tokens_out"]) -> None: ...
global___MsgExitPoolResponse = MsgExitPoolResponse

class MsgSwapAssets(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    POOL_ID_FIELD_NUMBER: builtins.int
    TOKEN_IN_FIELD_NUMBER: builtins.int
    TOKEN_OUT_DENOM_FIELD_NUMBER: builtins.int
    sender: typing.Text
    pool_id: builtins.int
    @property
    def token_in(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    token_out_denom: typing.Text
    def __init__(self,
        *,
        sender: typing.Text = ...,
        pool_id: builtins.int = ...,
        token_in: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        token_out_denom: typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token_in",b"token_in"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pool_id",b"pool_id","sender",b"sender","token_in",b"token_in","token_out_denom",b"token_out_denom"]) -> None: ...
global___MsgSwapAssets = MsgSwapAssets

class MsgSwapAssetsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOKEN_OUT_FIELD_NUMBER: builtins.int
    @property
    def token_out(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        token_out: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token_out",b"token_out"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["token_out",b"token_out"]) -> None: ...
global___MsgSwapAssetsResponse = MsgSwapAssetsResponse

class EventPoolJoined(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ADDRESS_FIELD_NUMBER: builtins.int
    POOL_ID_FIELD_NUMBER: builtins.int
    TOKENS_IN_FIELD_NUMBER: builtins.int
    POOL_SHARES_OUT_FIELD_NUMBER: builtins.int
    REM_COINS_FIELD_NUMBER: builtins.int
    address: typing.Text
    pool_id: builtins.int
    @property
    def tokens_in(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def pool_shares_out(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    @property
    def rem_coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        address: typing.Text = ...,
        pool_id: builtins.int = ...,
        tokens_in: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        pool_shares_out: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        rem_coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool_shares_out",b"pool_shares_out"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["address",b"address","pool_id",b"pool_id","pool_shares_out",b"pool_shares_out","rem_coins",b"rem_coins","tokens_in",b"tokens_in"]) -> None: ...
global___EventPoolJoined = EventPoolJoined

class EventPoolCreated(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CREATOR_FIELD_NUMBER: builtins.int
    POOL_ID_FIELD_NUMBER: builtins.int
    creator: typing.Text
    pool_id: builtins.int
    def __init__(self,
        *,
        creator: typing.Text = ...,
        pool_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["creator",b"creator","pool_id",b"pool_id"]) -> None: ...
global___EventPoolCreated = EventPoolCreated

class EventPoolExited(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ADDRESS_FIELD_NUMBER: builtins.int
    POOL_ID_FIELD_NUMBER: builtins.int
    POOL_SHARES_IN_FIELD_NUMBER: builtins.int
    TOKENS_OUT_FIELD_NUMBER: builtins.int
    address: typing.Text
    pool_id: builtins.int
    @property
    def pool_shares_in(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    @property
    def tokens_out(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        address: typing.Text = ...,
        pool_id: builtins.int = ...,
        pool_shares_in: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        tokens_out: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool_shares_in",b"pool_shares_in"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["address",b"address","pool_id",b"pool_id","pool_shares_in",b"pool_shares_in","tokens_out",b"tokens_out"]) -> None: ...
global___EventPoolExited = EventPoolExited

class EventAssetsSwapped(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ADDRESS_FIELD_NUMBER: builtins.int
    POOL_ID_FIELD_NUMBER: builtins.int
    TOKEN_IN_FIELD_NUMBER: builtins.int
    TOKEN_OUT_FIELD_NUMBER: builtins.int
    address: typing.Text
    pool_id: builtins.int
    @property
    def token_in(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    @property
    def token_out(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        address: typing.Text = ...,
        pool_id: builtins.int = ...,
        token_in: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        token_out: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token_in",b"token_in","token_out",b"token_out"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["address",b"address","pool_id",b"pool_id","token_in",b"token_in","token_out",b"token_out"]) -> None: ...
global___EventAssetsSwapped = EventAssetsSwapped



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Params(google.protobuf.message.Message):
    """Params defines the parameters for the oracle module."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VOTE_PERIOD_FIELD_NUMBER: builtins.int
    VOTE_THRESHOLD_FIELD_NUMBER: builtins.int
    REWARD_BAND_FIELD_NUMBER: builtins.int
    REWARD_DISTRIBUTION_WINDOW_FIELD_NUMBER: builtins.int
    WHITELIST_FIELD_NUMBER: builtins.int
    SLASH_FRACTION_FIELD_NUMBER: builtins.int
    SLASH_WINDOW_FIELD_NUMBER: builtins.int
    MIN_VALID_PER_WINDOW_FIELD_NUMBER: builtins.int
    vote_period: builtins.int
    vote_threshold: typing.Text
    reward_band: typing.Text
    reward_distribution_window: builtins.int
    @property
    def whitelist(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Pair]: ...
    slash_fraction: typing.Text
    slash_window: builtins.int
    min_valid_per_window: typing.Text
    def __init__(self,
        *,
        vote_period: builtins.int = ...,
        vote_threshold: typing.Text = ...,
        reward_band: typing.Text = ...,
        reward_distribution_window: builtins.int = ...,
        whitelist: typing.Optional[typing.Iterable[global___Pair]] = ...,
        slash_fraction: typing.Text = ...,
        slash_window: builtins.int = ...,
        min_valid_per_window: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["min_valid_per_window",b"min_valid_per_window","reward_band",b"reward_band","reward_distribution_window",b"reward_distribution_window","slash_fraction",b"slash_fraction","slash_window",b"slash_window","vote_period",b"vote_period","vote_threshold",b"vote_threshold","whitelist",b"whitelist"]) -> None: ...
global___Params = Params

class Pair(google.protobuf.message.Message):
    """Pair is the object that holds configuration of each pair."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NAME_FIELD_NUMBER: builtins.int
    TOBIN_TAX_FIELD_NUMBER: builtins.int
    name: typing.Text
    tobin_tax: typing.Text
    def __init__(self,
        *,
        name: typing.Text = ...,
        tobin_tax: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name",b"name","tobin_tax",b"tobin_tax"]) -> None: ...
global___Pair = Pair

class AggregateExchangeRatePrevote(google.protobuf.message.Message):
    """struct for aggregate prevoting on the ExchangeRateVote.
    The purpose of aggregate prevote is to hide vote exchange rates with hash
    which is formatted as hex string in SHA256("{salt}:({pair},{exchange_rate})|...,({pair},{exchange_rate}):{voter}")
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HASH_FIELD_NUMBER: builtins.int
    VOTER_FIELD_NUMBER: builtins.int
    SUBMIT_BLOCK_FIELD_NUMBER: builtins.int
    hash: typing.Text
    voter: typing.Text
    submit_block: builtins.int
    def __init__(self,
        *,
        hash: typing.Text = ...,
        voter: typing.Text = ...,
        submit_block: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["hash",b"hash","submit_block",b"submit_block","voter",b"voter"]) -> None: ...
global___AggregateExchangeRatePrevote = AggregateExchangeRatePrevote

class AggregateExchangeRateVote(google.protobuf.message.Message):
    """MsgAggregateExchangeRateVote - struct for voting on
    the exchange rates different assets.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EXCHANGE_RATE_TUPLES_FIELD_NUMBER: builtins.int
    VOTER_FIELD_NUMBER: builtins.int
    @property
    def exchange_rate_tuples(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ExchangeRateTuple]: ...
    voter: typing.Text
    def __init__(self,
        *,
        exchange_rate_tuples: typing.Optional[typing.Iterable[global___ExchangeRateTuple]] = ...,
        voter: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["exchange_rate_tuples",b"exchange_rate_tuples","voter",b"voter"]) -> None: ...
global___AggregateExchangeRateVote = AggregateExchangeRateVote

class ExchangeRateTuple(google.protobuf.message.Message):
    """ExchangeRateTuple - struct to store interpreted exchange rates data to store"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    EXCHANGE_RATE_FIELD_NUMBER: builtins.int
    pair: typing.Text
    exchange_rate: typing.Text
    def __init__(self,
        *,
        pair: typing.Text = ...,
        exchange_rate: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["exchange_rate",b"exchange_rate","pair",b"pair"]) -> None: ...
global___ExchangeRateTuple = ExchangeRateTuple



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Coin(google.protobuf.message.Message):
    """Coin defines a token with a denomination and an amount.

    NOTE: The amount field is an Int which implements the custom method
    signatures required by gogoproto.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DENOM_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    denom: typing.Text
    amount: typing.Text
    def __init__(self,
        *,
        denom: typing.Text = ...,
        amount: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","denom",b"denom"]) -> None: ...
global___Coin = Coin

class DecCoin(google.protobuf.message.Message):
    """DecCoin defines a token with a denomination and a decimal amount.

    NOTE: The amount field is an Dec which implements the custom method
    signatures required by gogoproto.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DENOM_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    denom: typing.Text
    amount: typing.Text
    def __init__(self,
        *,
        denom: typing.Text = ...,
        amount: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","denom",b"denom"]) -> None: ...
global___DecCoin = DecCoin

class IntProto(google.protobuf.message.Message):
    """IntProto defines a Protobuf wrapper around an Int object."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INT_FIELD_NUMBER: builtins.int
    int: typing.Text
    def __init__(self,
        *,
        int: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["int",b"int"]) -> None: ...
global___IntProto = IntProto

class DecProto(google.protobuf.message.Message):
    """DecProto defines a Protobuf wrapper around a Dec object."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DEC_FIELD_NUMBER: builtins.int
    dec: typing.Text
    def __init__(self,
        *,
        dec: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dec",b"dec"]) -> None: ...
global___DecProto = DecProto



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from cosmos.feegrant.v1beta1 import query_pb2 as cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2


class QueryStub(object):
    """Query defines the gRPC querier service.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Allowance = channel.unary_unary(
                '/cosmos.feegrant.v1beta1.Query/Allowance',
                request_serializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowanceRequest.SerializeToString,
                response_deserializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowanceResponse.FromString,
                )
        self.Allowances = channel.unary_unary(
                '/cosmos.feegrant.v1beta1.Query/Allowances',
                request_serializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesRequest.SerializeToString,
                response_deserializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesResponse.FromString,
                )
        self.AllowancesByGranter = channel.unary_unary(
                '/cosmos.feegrant.v1beta1.Query/AllowancesByGranter',
                request_serializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesByGranterRequest.SerializeToString,
                response_deserializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesByGranterResponse.FromString,
                )


class QueryServicer(object):
    """Query defines the gRPC querier service.
    """

    def Allowance(self, request, context):
        """Allowance returns fee granted to the grantee by the granter.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Allowances(self, request, context):
        """Allowances returns all the grants for address.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AllowancesByGranter(self, request, context):
        """AllowancesByGranter returns all the grants given by an address
        Since v0.46
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QueryServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Allowance': grpc.unary_unary_rpc_method_handler(
                    servicer.Allowance,
                    request_deserializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowanceRequest.FromString,
                    response_serializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowanceResponse.SerializeToString,
            ),
            'Allowances': grpc.unary_unary_rpc_method_handler(
                    servicer.Allowances,
                    request_deserializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesRequest.FromString,
                    response_serializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesResponse.SerializeToString,
            ),
            'AllowancesByGranter': grpc.unary_unary_rpc_method_handler(
                    servicer.AllowancesByGranter,
                    request_deserializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesByGranterRequest.FromString,
                    response_serializer=cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesByGranterResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cosmos.feegrant.v1beta1.Query', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Query(object):
    """Query defines the gRPC querier service.
    """

    @staticmethod
    def Allowance(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cosmos.feegrant.v1beta1.Query/Allowance',
            cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowanceRequest.SerializeToString,
            cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowanceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Allowances(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cosmos.feegrant.v1beta1.Query/Allowances',
            cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesRequest.SerializeToString,
            cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AllowancesByGranter(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cosmos.feegrant.v1beta1.Query/AllowancesByGranter',
            cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesByGranterRequest.SerializeToString,
            cosmos_dot_feegrant_dot_v1beta1_dot_query__pb2.QueryAllowancesByGranterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import lockup.v1.tx_pb2

class MsgStub:
    """Msg defines the Msg service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    LockTokens: grpc.UnaryUnaryMultiCallable[
        lockup.v1.tx_pb2.MsgLockTokens,
        lockup.v1.tx_pb2.MsgLockTokensResponse]
    """LockTokens lock tokens"""

    InitiateUnlock: grpc.UnaryUnaryMultiCallable[
        lockup.v1.tx_pb2.MsgInitiateUnlock,
        lockup.v1.tx_pb2.MsgInitiateUnlockResponse]

    Unlock: grpc.UnaryUnaryMultiCallable[
        lockup.v1.tx_pb2.MsgUnlock,
        lockup.v1.tx_pb2.MsgUnlockResponse]


class MsgServicer(metaclass=abc.ABCMeta):
    """Msg defines the Msg service."""
    @abc.abstractmethod
    def LockTokens(self,
        request: lockup.v1.tx_pb2.MsgLockTokens,
        context: grpc.ServicerContext,
    ) -> lockup.v1.tx_pb2.MsgLockTokensResponse:
        """LockTokens lock tokens"""
        pass

    @abc.abstractmethod
    def InitiateUnlock(self,
        request: lockup.v1.tx_pb2.MsgInitiateUnlock,
        context: grpc.ServicerContext,
    ) -> lockup.v1.tx_pb2.MsgInitiateUnlockResponse: ...

    @abc.abstractmethod
    def Unlock(self,
        request: lockup.v1.tx_pb2.MsgUnlock,
        context: grpc.ServicerContext,
    ) -> lockup.v1.tx_pb2.MsgUnlockResponse: ...


def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: oracle/v1beta1/oracle.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1boracle/v1beta1/oracle.proto\x12\x15nibiru.oracle.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\"\x99\x05\n\x06Params\x12+\n\x0bvote_period\x18\x01 \x01(\x04\x42\x16\xf2\xde\x1f\x12yaml:\"vote_period\"\x12_\n\x0evote_threshold\x18\x02 \x01(\tBG\xf2\xde\x1f\x15yaml:\"vote_threshold\"\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12Y\n\x0breward_band\x18\x03 \x01(\tBD\xf2\xde\x1f\x12yaml:\"reward_band\"\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12I\n\x1areward_distribution_window\x18\x04 \x01(\x04\x42%\xf2\xde\x1f!yaml:\"reward_distribution_window\"\x12T\n\twhitelist\x18\x05 \x03(\x0b\x32\x1b.nibiru.oracle.v1beta1.PairB$\xf2\xde\x1f\x10yaml:\"whitelist\"\xaa\xdf\x1f\x08PairList\xc8\xde\x1f\x00\x12_\n\x0eslash_fraction\x18\x06 \x01(\tBG\xf2\xde\x1f\x15yaml:\"slash_fraction\"\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12-\n\x0cslash_window\x18\x07 \x01(\x04\x42\x17\xf2\xde\x1f\x13yaml:\"slash_window\"\x12k\n\x14min_valid_per_window\x18\x08 \x01(\tBM\xf2\xde\x1f\x1byaml:\"min_valid_per_window\"\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00:\x08\xe8\xa0\x1f\x01\x98\xa0\x1f\x00\"\x8a\x01\n\x04Pair\x12\x1d\n\x04name\x18\x01 \x01(\tB\x0f\xf2\xde\x1f\x0byaml:\"name\"\x12U\n\ttobin_tax\x18\x02 \x01(\tBB\xf2\xde\x1f\x10yaml:\"tobin_tax\"\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00:\x0c\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\x98\xa0\x1f\x00\"\x9b\x01\n\x1c\x41ggregateExchangeRatePrevote\x12\x1d\n\x04hash\x18\x01 \x01(\tB\x0f\xf2\xde\x1f\x0byaml:\"hash\"\x12\x1f\n\x05voter\x18\x02 \x01(\tB\x10\xf2\xde\x1f\x0cyaml:\"voter\"\x12-\n\x0csubmit_block\x18\x03 \x01(\x04\x42\x17\xf2\xde\x1f\x13yaml:\"submit_block\":\x0c\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\x98\xa0\x1f\x00\"\xce\x01\n\x19\x41ggregateExchangeRateVote\x12\x81\x01\n\x14\x65xchange_rate_tuples\x18\x01 \x03(\x0b\x32(.nibiru.oracle.v1beta1.ExchangeRateTupleB9\xf2\xde\x1f\x1byaml:\"exchange_rate_tuples\"\xaa\xdf\x1f\x12\x45xchangeRateTuples\xc8\xde\x1f\x00\x12\x1f\n\x05voter\x18\x02 \x01(\tB\x10\xf2\xde\x1f\x0cyaml:\"voter\":\x0c\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\x98\xa0\x1f\x00\"\x9f\x01\n\x11\x45xchangeRateTuple\x12\x1d\n\x04pair\x18\x01 \x01(\tB\x0f\xf2\xde\x1f\x0byaml:\"pair\"\x12]\n\rexchange_rate\x18\x02 \x01(\tBF\xf2\xde\x1f\x14yaml:\"exchange_rate\"\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00:\x0c\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\x98\xa0\x1f\x00\x42.Z,github.com/NibiruChain/nibiru/x/oracle/typesb\x06proto3')



_PARAMS = DESCRIPTOR.message_types_by_name['Params']
_PAIR = DESCRIPTOR.message_types_by_name['Pair']
_AGGREGATEEXCHANGERATEPREVOTE = DESCRIPTOR.message_types_by_name['AggregateExchangeRatePrevote']
_AGGREGATEEXCHANGERATEVOTE = DESCRIPTOR.message_types_by_name['AggregateExchangeRateVote']
_EXCHANGERATETUPLE = DESCRIPTOR.message_types_by_name['ExchangeRateTuple']
Params = _reflection.GeneratedProtocolMessageType('Params', (_message.Message,), {
  'DESCRIPTOR' : _PARAMS,
  '__module__' : 'oracle.v1beta1.oracle_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.Params)
  })
_sym_db.RegisterMessage(Params)

Pair = _reflection.GeneratedProtocolMessageType('Pair', (_message.Message,), {
  'DESCRIPTOR' : _PAIR,
  '__module__' : 'oracle.v1beta1.oracle_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.Pair)
  })
_sym_db.RegisterMessage(Pair)

AggregateExchangeRatePrevote = _reflection.GeneratedProtocolMessageType('AggregateExchangeRatePrevote', (_message.Message,), {
  'DESCRIPTOR' : _AGGREGATEEXCHANGERATEPREVOTE,
  '__module__' : 'oracle.v1beta1.oracle_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.AggregateExchangeRatePrevote)
  })
_sym_db.RegisterMessage(AggregateExchangeRatePrevote)

AggregateExchangeRateVote = _reflection.GeneratedProtocolMessageType('AggregateExchangeRateVote', (_message.Message,), {
  'DESCRIPTOR' : _AGGREGATEEXCHANGERATEVOTE,
  '__module__' : 'oracle.v1beta1.oracle_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.AggregateExchangeRateVote)
  })
_sym_db.RegisterMessage(AggregateExchangeRateVote)

ExchangeRateTuple = _reflection.GeneratedProtocolMessageType('ExchangeRateTuple', (_message.Message,), {
  'DESCRIPTOR' : _EXCHANGERATETUPLE,
  '__module__' : 'oracle.v1beta1.oracle_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.ExchangeRateTuple)
  })
_sym_db.RegisterMessage(ExchangeRateTuple)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/NibiruChain/nibiru/x/oracle/types'
  _PARAMS.fields_by_name['vote_period']._options = None
  _PARAMS.fields_by_name['vote_period']._serialized_options = b'\362\336\037\022yaml:\"vote_period\"'
  _PARAMS.fields_by_name['vote_threshold']._options = None
  _PARAMS.fields_by_name['vote_threshold']._serialized_options = b'\362\336\037\025yaml:\"vote_threshold\"\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['reward_band']._options = None
  _PARAMS.fields_by_name['reward_band']._serialized_options = b'\362\336\037\022yaml:\"reward_band\"\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['reward_distribution_window']._options = None
  _PARAMS.fields_by_name['reward_distribution_window']._serialized_options = b'\362\336\037!yaml:\"reward_distribution_window\"'
  _PARAMS.fields_by_name['whitelist']._options = None
  _PARAMS.fields_by_name['whitelist']._serialized_options = b'\362\336\037\020yaml:\"whitelist\"\252\337\037\010PairList\310\336\037\000'
  _PARAMS.fields_by_name['slash_fraction']._options = None
  _PARAMS.fields_by_name['slash_fraction']._serialized_options = b'\362\336\037\025yaml:\"slash_fraction\"\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS.fields_by_name['slash_window']._options = None
  _PARAMS.fields_by_name['slash_window']._serialized_options = b'\362\336\037\023yaml:\"slash_window\"'
  _PARAMS.fields_by_name['min_valid_per_window']._options = None
  _PARAMS.fields_by_name['min_valid_per_window']._serialized_options = b'\362\336\037\033yaml:\"min_valid_per_window\"\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PARAMS._options = None
  _PARAMS._serialized_options = b'\350\240\037\001\230\240\037\000'
  _PAIR.fields_by_name['name']._options = None
  _PAIR.fields_by_name['name']._serialized_options = b'\362\336\037\013yaml:\"name\"'
  _PAIR.fields_by_name['tobin_tax']._options = None
  _PAIR.fields_by_name['tobin_tax']._serialized_options = b'\362\336\037\020yaml:\"tobin_tax\"\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _PAIR._options = None
  _PAIR._serialized_options = b'\350\240\037\000\210\240\037\000\230\240\037\000'
  _AGGREGATEEXCHANGERATEPREVOTE.fields_by_name['hash']._options = None
  _AGGREGATEEXCHANGERATEPREVOTE.fields_by_name['hash']._serialized_options = b'\362\336\037\013yaml:\"hash\"'
  _AGGREGATEEXCHANGERATEPREVOTE.fields_by_name['voter']._options = None
  _AGGREGATEEXCHANGERATEPREVOTE.fields_by_name['voter']._serialized_options = b'\362\336\037\014yaml:\"voter\"'
  _AGGREGATEEXCHANGERATEPREVOTE.fields_by_name['submit_block']._options = None
  _AGGREGATEEXCHANGERATEPREVOTE.fields_by_name['submit_block']._serialized_options = b'\362\336\037\023yaml:\"submit_block\"'
  _AGGREGATEEXCHANGERATEPREVOTE._options = None
  _AGGREGATEEXCHANGERATEPREVOTE._serialized_options = b'\350\240\037\000\210\240\037\000\230\240\037\000'
  _AGGREGATEEXCHANGERATEVOTE.fields_by_name['exchange_rate_tuples']._options = None
  _AGGREGATEEXCHANGERATEVOTE.fields_by_name['exchange_rate_tuples']._serialized_options = b'\362\336\037\033yaml:\"exchange_rate_tuples\"\252\337\037\022ExchangeRateTuples\310\336\037\000'
  _AGGREGATEEXCHANGERATEVOTE.fields_by_name['voter']._options = None
  _AGGREGATEEXCHANGERATEVOTE.fields_by_name['voter']._serialized_options = b'\362\336\037\014yaml:\"voter\"'
  _AGGREGATEEXCHANGERATEVOTE._options = None
  _AGGREGATEEXCHANGERATEVOTE._serialized_options = b'\350\240\037\000\210\240\037\000\230\240\037\000'
  _EXCHANGERATETUPLE.fields_by_name['pair']._options = None
  _EXCHANGERATETUPLE.fields_by_name['pair']._serialized_options = b'\362\336\037\013yaml:\"pair\"'
  _EXCHANGERATETUPLE.fields_by_name['exchange_rate']._options = None
  _EXCHANGERATETUPLE.fields_by_name['exchange_rate']._serialized_options = b'\362\336\037\024yaml:\"exchange_rate\"\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _EXCHANGERATETUPLE._options = None
  _EXCHANGERATETUPLE._serialized_options = b'\350\240\037\000\210\240\037\000\230\240\037\000'
  _PARAMS._serialized_start=109
  _PARAMS._serialized_end=774
  _PAIR._serialized_start=777
  _PAIR._serialized_end=915
  _AGGREGATEEXCHANGERATEPREVOTE._serialized_start=918
  _AGGREGATEEXCHANGERATEPREVOTE._serialized_end=1073
  _AGGREGATEEXCHANGERATEVOTE._serialized_start=1076
  _AGGREGATEEXCHANGERATEVOTE._serialized_end=1282
  _EXCHANGERATETUPLE._serialized_start=1285
  _EXCHANGERATETUPLE._serialized_end=1444
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.bank.v1beta1.bank_pb2
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgSend(google.protobuf.message.Message):
    """MsgSend represents a message to send coins from one account to another."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FROM_ADDRESS_FIELD_NUMBER: builtins.int
    TO_ADDRESS_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    from_address: typing.Text
    to_address: typing.Text
    @property
    def amount(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        from_address: typing.Text = ...,
        to_address: typing.Text = ...,
        amount: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","from_address",b"from_address","to_address",b"to_address"]) -> None: ...
global___MsgSend = MsgSend

class MsgSendResponse(google.protobuf.message.Message):
    """MsgSendResponse defines the Msg/Send response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgSendResponse = MsgSendResponse

class MsgMultiSend(google.protobuf.message.Message):
    """MsgMultiSend represents an arbitrary multi-in, multi-out send message."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INPUTS_FIELD_NUMBER: builtins.int
    OUTPUTS_FIELD_NUMBER: builtins.int
    @property
    def inputs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.bank.v1beta1.bank_pb2.Input]: ...
    @property
    def outputs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.bank.v1beta1.bank_pb2.Output]: ...
    def __init__(self,
        *,
        inputs: typing.Optional[typing.Iterable[cosmos.bank.v1beta1.bank_pb2.Input]] = ...,
        outputs: typing.Optional[typing.Iterable[cosmos.bank.v1beta1.bank_pb2.Output]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["inputs",b"inputs","outputs",b"outputs"]) -> None: ...
global___MsgMultiSend = MsgMultiSend

class MsgMultiSendResponse(google.protobuf.message.Message):
    """MsgMultiSendResponse defines the Msg/MultiSend response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgMultiSendResponse = MsgMultiSendResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class PoolParams(google.protobuf.message.Message):
    """Configuration parameters for the pool."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SWAP_FEE_FIELD_NUMBER: builtins.int
    EXIT_FEE_FIELD_NUMBER: builtins.int
    swap_fee: typing.Text
    exit_fee: typing.Text
    def __init__(self,
        *,
        swap_fee: typing.Text = ...,
        exit_fee: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["exit_fee",b"exit_fee","swap_fee",b"swap_fee"]) -> None: ...
global___PoolParams = PoolParams

class PoolAsset(google.protobuf.message.Message):
    """Which assets the pool contains."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOKEN_FIELD_NUMBER: builtins.int
    WEIGHT_FIELD_NUMBER: builtins.int
    @property
    def token(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """Coins we are talking about,
        the denomination must be unique amongst all PoolAssets for this pool.
        """
        pass
    weight: typing.Text
    """Weight that is not normalized. This weight must be less than 2^50"""

    def __init__(self,
        *,
        token: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        weight: typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["token",b"token"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["token",b"token","weight",b"weight"]) -> None: ...
global___PoolAsset = PoolAsset

class Pool(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ID_FIELD_NUMBER: builtins.int
    ADDRESS_FIELD_NUMBER: builtins.int
    POOL_PARAMS_FIELD_NUMBER: builtins.int
    POOL_ASSETS_FIELD_NUMBER: builtins.int
    TOTAL_WEIGHT_FIELD_NUMBER: builtins.int
    TOTAL_SHARES_FIELD_NUMBER: builtins.int
    id: builtins.int
    """The pool id."""

    address: typing.Text
    """The pool account address."""

    @property
    def pool_params(self) -> global___PoolParams:
        """Fees and other pool-specific parameters."""
        pass
    @property
    def pool_assets(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___PoolAsset]:
        """These are assumed to be sorted by denomiation.
        They contain the pool asset and the information about the weight
        """
        pass
    total_weight: typing.Text
    """sum of all non-normalized pool weights"""

    @property
    def total_shares(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """sum of all LP tokens sent out"""
        pass
    def __init__(self,
        *,
        id: builtins.int = ...,
        address: typing.Text = ...,
        pool_params: typing.Optional[global___PoolParams] = ...,
        pool_assets: typing.Optional[typing.Iterable[global___PoolAsset]] = ...,
        total_weight: typing.Text = ...,
        total_shares: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pool_params",b"pool_params","total_shares",b"total_shares"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["address",b"address","id",b"id","pool_assets",b"pool_assets","pool_params",b"pool_params","total_shares",b"total_shares","total_weight",b"total_weight"]) -> None: ...
global___Pool = Pool



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import oracle.v1beta1.oracle_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class GenesisState(google.protobuf.message.Message):
    """GenesisState defines the oracle module's genesis state."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_FIELD_NUMBER: builtins.int
    FEEDER_DELEGATIONS_FIELD_NUMBER: builtins.int
    EXCHANGE_RATES_FIELD_NUMBER: builtins.int
    MISS_COUNTERS_FIELD_NUMBER: builtins.int
    AGGREGATE_EXCHANGE_RATE_PREVOTES_FIELD_NUMBER: builtins.int
    AGGREGATE_EXCHANGE_RATE_VOTES_FIELD_NUMBER: builtins.int
    TOBIN_TAXES_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> oracle.v1beta1.oracle_pb2.Params: ...
    @property
    def feeder_delegations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___FeederDelegation]: ...
    @property
    def exchange_rates(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[oracle.v1beta1.oracle_pb2.ExchangeRateTuple]: ...
    @property
    def miss_counters(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MissCounter]: ...
    @property
    def aggregate_exchange_rate_prevotes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[oracle.v1beta1.oracle_pb2.AggregateExchangeRatePrevote]: ...
    @property
    def aggregate_exchange_rate_votes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[oracle.v1beta1.oracle_pb2.AggregateExchangeRateVote]: ...
    @property
    def tobin_taxes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___TobinTax]: ...
    def __init__(self,
        *,
        params: typing.Optional[oracle.v1beta1.oracle_pb2.Params] = ...,
        feeder_delegations: typing.Optional[typing.Iterable[global___FeederDelegation]] = ...,
        exchange_rates: typing.Optional[typing.Iterable[oracle.v1beta1.oracle_pb2.ExchangeRateTuple]] = ...,
        miss_counters: typing.Optional[typing.Iterable[global___MissCounter]] = ...,
        aggregate_exchange_rate_prevotes: typing.Optional[typing.Iterable[oracle.v1beta1.oracle_pb2.AggregateExchangeRatePrevote]] = ...,
        aggregate_exchange_rate_votes: typing.Optional[typing.Iterable[oracle.v1beta1.oracle_pb2.AggregateExchangeRateVote]] = ...,
        tobin_taxes: typing.Optional[typing.Iterable[global___TobinTax]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregate_exchange_rate_prevotes",b"aggregate_exchange_rate_prevotes","aggregate_exchange_rate_votes",b"aggregate_exchange_rate_votes","exchange_rates",b"exchange_rates","feeder_delegations",b"feeder_delegations","miss_counters",b"miss_counters","params",b"params","tobin_taxes",b"tobin_taxes"]) -> None: ...
global___GenesisState = GenesisState

class FeederDelegation(google.protobuf.message.Message):
    """FeederDelegation is the address for where oracle feeder authority are
    delegated to. By default this struct is only used at genesis to feed in
    default feeder addresses.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FEEDER_ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    feeder_address: typing.Text
    validator_address: typing.Text
    def __init__(self,
        *,
        feeder_address: typing.Text = ...,
        validator_address: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["feeder_address",b"feeder_address","validator_address",b"validator_address"]) -> None: ...
global___FeederDelegation = FeederDelegation

class MissCounter(google.protobuf.message.Message):
    """MissCounter defines an miss counter and validator address pair used in
    oracle module's genesis state
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    MISS_COUNTER_FIELD_NUMBER: builtins.int
    validator_address: typing.Text
    miss_counter: builtins.int
    def __init__(self,
        *,
        validator_address: typing.Text = ...,
        miss_counter: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["miss_counter",b"miss_counter","validator_address",b"validator_address"]) -> None: ...
global___MissCounter = MissCounter

class TobinTax(google.protobuf.message.Message):
    """TobinTax defines a pair and tobin_tax pair used in
    oracle module's genesis state
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_FIELD_NUMBER: builtins.int
    TOBIN_TAX_FIELD_NUMBER: builtins.int
    pair: typing.Text
    tobin_tax: typing.Text
    def __init__(self,
        *,
        pair: typing.Text = ...,
        tobin_tax: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pair",b"pair","tobin_tax",b"tobin_tax"]) -> None: ...
global___TobinTax = TobinTax



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ParameterChangeProposal(google.protobuf.message.Message):
    """ParameterChangeProposal defines a proposal to change one or more parameters."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TITLE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    CHANGES_FIELD_NUMBER: builtins.int
    title: typing.Text
    description: typing.Text
    @property
    def changes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ParamChange]: ...
    def __init__(self,
        *,
        title: typing.Text = ...,
        description: typing.Text = ...,
        changes: typing.Optional[typing.Iterable[global___ParamChange]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["changes",b"changes","description",b"description","title",b"title"]) -> None: ...
global___ParameterChangeProposal = ParameterChangeProposal

class ParamChange(google.protobuf.message.Message):
    """ParamChange defines an individual parameter change, for use in
    ParameterChangeProposal.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SUBSPACE_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    subspace: typing.Text
    key: typing.Text
    value: typing.Text
    def __init__(self,
        *,
        subspace: typing.Text = ...,
        key: typing.Text = ...,
        value: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key",b"key","subspace",b"subspace","value",b"value"]) -> None: ...
global___ParamChange = ParamChange



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: oracle/v1beta1/genesis.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from oracle.v1beta1 import oracle_pb2 as oracle_dot_v1beta1_dot_oracle__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1coracle/v1beta1/genesis.proto\x12\x15nibiru.oracle.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1boracle/v1beta1/oracle.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\"\xad\x04\n\x0cGenesisState\x12\x33\n\x06params\x18\x01 \x01(\x0b\x32\x1d.nibiru.oracle.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\x12I\n\x12\x66\x65\x65\x64\x65r_delegations\x18\x02 \x03(\x0b\x32\'.nibiru.oracle.v1beta1.FeederDelegationB\x04\xc8\xde\x1f\x00\x12\\\n\x0e\x65xchange_rates\x18\x03 \x03(\x0b\x32(.nibiru.oracle.v1beta1.ExchangeRateTupleB\x1a\xaa\xdf\x1f\x12\x45xchangeRateTuples\xc8\xde\x1f\x00\x12?\n\rmiss_counters\x18\x04 \x03(\x0b\x32\".nibiru.oracle.v1beta1.MissCounterB\x04\xc8\xde\x1f\x00\x12\x63\n aggregate_exchange_rate_prevotes\x18\x05 \x03(\x0b\x32\x33.nibiru.oracle.v1beta1.AggregateExchangeRatePrevoteB\x04\xc8\xde\x1f\x00\x12]\n\x1d\x61ggregate_exchange_rate_votes\x18\x06 \x03(\x0b\x32\x30.nibiru.oracle.v1beta1.AggregateExchangeRateVoteB\x04\xc8\xde\x1f\x00\x12:\n\x0btobin_taxes\x18\x07 \x03(\x0b\x32\x1f.nibiru.oracle.v1beta1.TobinTaxB\x04\xc8\xde\x1f\x00\"E\n\x10\x46\x65\x65\x64\x65rDelegation\x12\x16\n\x0e\x66\x65\x65\x64\x65r_address\x18\x01 \x01(\t\x12\x19\n\x11validator_address\x18\x02 \x01(\t\">\n\x0bMissCounter\x12\x19\n\x11validator_address\x18\x01 \x01(\t\x12\x14\n\x0cmiss_counter\x18\x02 \x01(\x04\"[\n\x08TobinTax\x12\x0c\n\x04pair\x18\x01 \x01(\t\x12\x41\n\ttobin_tax\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x42.Z,github.com/NibiruChain/nibiru/x/oracle/typesb\x06proto3')



_GENESISSTATE = DESCRIPTOR.message_types_by_name['GenesisState']
_FEEDERDELEGATION = DESCRIPTOR.message_types_by_name['FeederDelegation']
_MISSCOUNTER = DESCRIPTOR.message_types_by_name['MissCounter']
_TOBINTAX = DESCRIPTOR.message_types_by_name['TobinTax']
GenesisState = _reflection.GeneratedProtocolMessageType('GenesisState', (_message.Message,), {
  'DESCRIPTOR' : _GENESISSTATE,
  '__module__' : 'oracle.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.GenesisState)
  })
_sym_db.RegisterMessage(GenesisState)

FeederDelegation = _reflection.GeneratedProtocolMessageType('FeederDelegation', (_message.Message,), {
  'DESCRIPTOR' : _FEEDERDELEGATION,
  '__module__' : 'oracle.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.FeederDelegation)
  })
_sym_db.RegisterMessage(FeederDelegation)

MissCounter = _reflection.GeneratedProtocolMessageType('MissCounter', (_message.Message,), {
  'DESCRIPTOR' : _MISSCOUNTER,
  '__module__' : 'oracle.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.MissCounter)
  })
_sym_db.RegisterMessage(MissCounter)

TobinTax = _reflection.GeneratedProtocolMessageType('TobinTax', (_message.Message,), {
  'DESCRIPTOR' : _TOBINTAX,
  '__module__' : 'oracle.v1beta1.genesis_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.TobinTax)
  })
_sym_db.RegisterMessage(TobinTax)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/NibiruChain/nibiru/x/oracle/types'
  _GENESISSTATE.fields_by_name['params']._options = None
  _GENESISSTATE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['feeder_delegations']._options = None
  _GENESISSTATE.fields_by_name['feeder_delegations']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['exchange_rates']._options = None
  _GENESISSTATE.fields_by_name['exchange_rates']._serialized_options = b'\252\337\037\022ExchangeRateTuples\310\336\037\000'
  _GENESISSTATE.fields_by_name['miss_counters']._options = None
  _GENESISSTATE.fields_by_name['miss_counters']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['aggregate_exchange_rate_prevotes']._options = None
  _GENESISSTATE.fields_by_name['aggregate_exchange_rate_prevotes']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['aggregate_exchange_rate_votes']._options = None
  _GENESISSTATE.fields_by_name['aggregate_exchange_rate_votes']._serialized_options = b'\310\336\037\000'
  _GENESISSTATE.fields_by_name['tobin_taxes']._options = None
  _GENESISSTATE.fields_by_name['tobin_taxes']._serialized_options = b'\310\336\037\000'
  _TOBINTAX.fields_by_name['tobin_tax']._options = None
  _TOBINTAX.fields_by_name['tobin_tax']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _GENESISSTATE._serialized_start=139
  _GENESISSTATE._serialized_end=696
  _FEEDERDELEGATION._serialized_start=698
  _FEEDERDELEGATION._serialized_end=767
  _MISSCOUNTER._serialized_start=769
  _MISSCOUNTER._serialized_end=831
  _TOBINTAX._serialized_start=833
  _TOBINTAX._serialized_end=924
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.abci.v1beta1.abci_pb2
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.tx.v1beta1.tx_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tendermint.types.block_pb2
import tendermint.types.types_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _OrderBy:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _OrderByEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_OrderBy.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ORDER_BY_UNSPECIFIED: _OrderBy.ValueType  # 0
    """ORDER_BY_UNSPECIFIED specifies an unknown sorting order. OrderBy defaults to ASC in this case."""

    ORDER_BY_ASC: _OrderBy.ValueType  # 1
    """ORDER_BY_ASC defines ascending order"""

    ORDER_BY_DESC: _OrderBy.ValueType  # 2
    """ORDER_BY_DESC defines descending order"""

class OrderBy(_OrderBy, metaclass=_OrderByEnumTypeWrapper):
    """OrderBy defines the sorting order"""
    pass

ORDER_BY_UNSPECIFIED: OrderBy.ValueType  # 0
"""ORDER_BY_UNSPECIFIED specifies an unknown sorting order. OrderBy defaults to ASC in this case."""

ORDER_BY_ASC: OrderBy.ValueType  # 1
"""ORDER_BY_ASC defines ascending order"""

ORDER_BY_DESC: OrderBy.ValueType  # 2
"""ORDER_BY_DESC defines descending order"""

global___OrderBy = OrderBy


class _BroadcastMode:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _BroadcastModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_BroadcastMode.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    BROADCAST_MODE_UNSPECIFIED: _BroadcastMode.ValueType  # 0
    """zero-value for mode ordering"""

    BROADCAST_MODE_BLOCK: _BroadcastMode.ValueType  # 1
    """BROADCAST_MODE_BLOCK defines a tx broadcasting mode where the client waits for
    the tx to be committed in a block.
    """

    BROADCAST_MODE_SYNC: _BroadcastMode.ValueType  # 2
    """BROADCAST_MODE_SYNC defines a tx broadcasting mode where the client waits for
    a CheckTx execution response only.
    """

    BROADCAST_MODE_ASYNC: _BroadcastMode.ValueType  # 3
    """BROADCAST_MODE_ASYNC defines a tx broadcasting mode where the client returns
    immediately.
    """

class BroadcastMode(_BroadcastMode, metaclass=_BroadcastModeEnumTypeWrapper):
    """BroadcastMode specifies the broadcast mode for the TxService.Broadcast RPC method."""
    pass

BROADCAST_MODE_UNSPECIFIED: BroadcastMode.ValueType  # 0
"""zero-value for mode ordering"""

BROADCAST_MODE_BLOCK: BroadcastMode.ValueType  # 1
"""BROADCAST_MODE_BLOCK defines a tx broadcasting mode where the client waits for
the tx to be committed in a block.
"""

BROADCAST_MODE_SYNC: BroadcastMode.ValueType  # 2
"""BROADCAST_MODE_SYNC defines a tx broadcasting mode where the client waits for
a CheckTx execution response only.
"""

BROADCAST_MODE_ASYNC: BroadcastMode.ValueType  # 3
"""BROADCAST_MODE_ASYNC defines a tx broadcasting mode where the client returns
immediately.
"""

global___BroadcastMode = BroadcastMode


class GetTxsEventRequest(google.protobuf.message.Message):
    """GetTxsEventRequest is the request type for the Service.TxsByEvents
    RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EVENTS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    ORDER_BY_FIELD_NUMBER: builtins.int
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
        """events is the list of transaction event type."""
        pass
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines a pagination for the request."""
        pass
    order_by: global___OrderBy.ValueType
    def __init__(self,
        *,
        events: typing.Optional[typing.Iterable[typing.Text]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        order_by: global___OrderBy.ValueType = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["events",b"events","order_by",b"order_by","pagination",b"pagination"]) -> None: ...
global___GetTxsEventRequest = GetTxsEventRequest

class GetTxsEventResponse(google.protobuf.message.Message):
    """GetTxsEventResponse is the response type for the Service.TxsByEvents
    RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TXS_FIELD_NUMBER: builtins.int
    TX_RESPONSES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def txs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.tx.v1beta1.tx_pb2.Tx]:
        """txs is the list of queried transactions."""
        pass
    @property
    def tx_responses(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.abci.v1beta1.abci_pb2.TxResponse]:
        """tx_responses is the list of queried TxResponses."""
        pass
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines a pagination for the response."""
        pass
    def __init__(self,
        *,
        txs: typing.Optional[typing.Iterable[cosmos.tx.v1beta1.tx_pb2.Tx]] = ...,
        tx_responses: typing.Optional[typing.Iterable[cosmos.base.abci.v1beta1.abci_pb2.TxResponse]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","tx_responses",b"tx_responses","txs",b"txs"]) -> None: ...
global___GetTxsEventResponse = GetTxsEventResponse

class BroadcastTxRequest(google.protobuf.message.Message):
    """BroadcastTxRequest is the request type for the Service.BroadcastTxRequest
    RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TX_BYTES_FIELD_NUMBER: builtins.int
    MODE_FIELD_NUMBER: builtins.int
    tx_bytes: builtins.bytes
    """tx_bytes is the raw transaction."""

    mode: global___BroadcastMode.ValueType
    def __init__(self,
        *,
        tx_bytes: builtins.bytes = ...,
        mode: global___BroadcastMode.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["mode",b"mode","tx_bytes",b"tx_bytes"]) -> None: ...
global___BroadcastTxRequest = BroadcastTxRequest

class BroadcastTxResponse(google.protobuf.message.Message):
    """BroadcastTxResponse is the response type for the
    Service.BroadcastTx method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TX_RESPONSE_FIELD_NUMBER: builtins.int
    @property
    def tx_response(self) -> cosmos.base.abci.v1beta1.abci_pb2.TxResponse:
        """tx_response is the queried TxResponses."""
        pass
    def __init__(self,
        *,
        tx_response: typing.Optional[cosmos.base.abci.v1beta1.abci_pb2.TxResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["tx_response",b"tx_response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["tx_response",b"tx_response"]) -> None: ...
global___BroadcastTxResponse = BroadcastTxResponse

class SimulateRequest(google.protobuf.message.Message):
    """SimulateRequest is the request type for the Service.Simulate
    RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TX_FIELD_NUMBER: builtins.int
    TX_BYTES_FIELD_NUMBER: builtins.int
    @property
    def tx(self) -> cosmos.tx.v1beta1.tx_pb2.Tx:
        """tx is the transaction to simulate.
        Deprecated. Send raw tx bytes instead.
        """
        pass
    tx_bytes: builtins.bytes
    """tx_bytes is the raw transaction.

    Since: cosmos-sdk 0.43
    """

    def __init__(self,
        *,
        tx: typing.Optional[cosmos.tx.v1beta1.tx_pb2.Tx] = ...,
        tx_bytes: builtins.bytes = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["tx",b"tx"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["tx",b"tx","tx_bytes",b"tx_bytes"]) -> None: ...
global___SimulateRequest = SimulateRequest

class SimulateResponse(google.protobuf.message.Message):
    """SimulateResponse is the response type for the
    Service.SimulateRPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GAS_INFO_FIELD_NUMBER: builtins.int
    RESULT_FIELD_NUMBER: builtins.int
    @property
    def gas_info(self) -> cosmos.base.abci.v1beta1.abci_pb2.GasInfo:
        """gas_info is the information about gas used in the simulation."""
        pass
    @property
    def result(self) -> cosmos.base.abci.v1beta1.abci_pb2.Result:
        """result is the result of the simulation."""
        pass
    def __init__(self,
        *,
        gas_info: typing.Optional[cosmos.base.abci.v1beta1.abci_pb2.GasInfo] = ...,
        result: typing.Optional[cosmos.base.abci.v1beta1.abci_pb2.Result] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["gas_info",b"gas_info","result",b"result"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["gas_info",b"gas_info","result",b"result"]) -> None: ...
global___SimulateResponse = SimulateResponse

class GetTxRequest(google.protobuf.message.Message):
    """GetTxRequest is the request type for the Service.GetTx
    RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HASH_FIELD_NUMBER: builtins.int
    hash: typing.Text
    """hash is the tx hash to query, encoded as a hex string."""

    def __init__(self,
        *,
        hash: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["hash",b"hash"]) -> None: ...
global___GetTxRequest = GetTxRequest

class GetTxResponse(google.protobuf.message.Message):
    """GetTxResponse is the response type for the Service.GetTx method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TX_FIELD_NUMBER: builtins.int
    TX_RESPONSE_FIELD_NUMBER: builtins.int
    @property
    def tx(self) -> cosmos.tx.v1beta1.tx_pb2.Tx:
        """tx is the queried transaction."""
        pass
    @property
    def tx_response(self) -> cosmos.base.abci.v1beta1.abci_pb2.TxResponse:
        """tx_response is the queried TxResponses."""
        pass
    def __init__(self,
        *,
        tx: typing.Optional[cosmos.tx.v1beta1.tx_pb2.Tx] = ...,
        tx_response: typing.Optional[cosmos.base.abci.v1beta1.abci_pb2.TxResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["tx",b"tx","tx_response",b"tx_response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["tx",b"tx","tx_response",b"tx_response"]) -> None: ...
global___GetTxResponse = GetTxResponse

class GetBlockWithTxsRequest(google.protobuf.message.Message):
    """GetBlockWithTxsRequest is the request type for the Service.GetBlockWithTxs
    RPC method.

    Since: cosmos-sdk 0.45.2
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HEIGHT_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    height: builtins.int
    """height is the height of the block to query."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines a pagination for the request."""
        pass
    def __init__(self,
        *,
        height: builtins.int = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["height",b"height","pagination",b"pagination"]) -> None: ...
global___GetBlockWithTxsRequest = GetBlockWithTxsRequest

class GetBlockWithTxsResponse(google.protobuf.message.Message):
    """GetBlockWithTxsResponse is the response type for the Service.GetBlockWithTxs method.

    Since: cosmos-sdk 0.45.2
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TXS_FIELD_NUMBER: builtins.int
    BLOCK_ID_FIELD_NUMBER: builtins.int
    BLOCK_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def txs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.tx.v1beta1.tx_pb2.Tx]:
        """txs are the transactions in the block."""
        pass
    @property
    def block_id(self) -> tendermint.types.types_pb2.BlockID: ...
    @property
    def block(self) -> tendermint.types.block_pb2.Block: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines a pagination for the response."""
        pass
    def __init__(self,
        *,
        txs: typing.Optional[typing.Iterable[cosmos.tx.v1beta1.tx_pb2.Tx]] = ...,
        block_id: typing.Optional[tendermint.types.types_pb2.BlockID] = ...,
        block: typing.Optional[tendermint.types.block_pb2.Block] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["block",b"block","block_id",b"block_id","pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block",b"block","block_id",b"block_id","pagination",b"pagination","txs",b"txs"]) -> None: ...
global___GetBlockWithTxsResponse = GetBlockWithTxsResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.distribution.v1beta1.tx_pb2
import grpc

class MsgStub:
    """Msg defines the distribution Msg service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    SetWithdrawAddress: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.tx_pb2.MsgSetWithdrawAddress,
        cosmos.distribution.v1beta1.tx_pb2.MsgSetWithdrawAddressResponse]
    """SetWithdrawAddress defines a method to change the withdraw address
    for a delegator (or validator self-delegation).
    """

    WithdrawDelegatorReward: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.tx_pb2.MsgWithdrawDelegatorReward,
        cosmos.distribution.v1beta1.tx_pb2.MsgWithdrawDelegatorRewardResponse]
    """WithdrawDelegatorReward defines a method to withdraw rewards of delegator
    from a single validator.
    """

    WithdrawValidatorCommission: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.tx_pb2.MsgWithdrawValidatorCommission,
        cosmos.distribution.v1beta1.tx_pb2.MsgWithdrawValidatorCommissionResponse]
    """WithdrawValidatorCommission defines a method to withdraw the
    full commission to the validator address.
    """

    FundCommunityPool: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.tx_pb2.MsgFundCommunityPool,
        cosmos.distribution.v1beta1.tx_pb2.MsgFundCommunityPoolResponse]
    """FundCommunityPool defines a method to allow an account to directly
    fund the community pool.
    """


class MsgServicer(metaclass=abc.ABCMeta):
    """Msg defines the distribution Msg service."""
    @abc.abstractmethod
    def SetWithdrawAddress(self,
        request: cosmos.distribution.v1beta1.tx_pb2.MsgSetWithdrawAddress,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.tx_pb2.MsgSetWithdrawAddressResponse:
        """SetWithdrawAddress defines a method to change the withdraw address
        for a delegator (or validator self-delegation).
        """
        pass

    @abc.abstractmethod
    def WithdrawDelegatorReward(self,
        request: cosmos.distribution.v1beta1.tx_pb2.MsgWithdrawDelegatorReward,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.tx_pb2.MsgWithdrawDelegatorRewardResponse:
        """WithdrawDelegatorReward defines a method to withdraw rewards of delegator
        from a single validator.
        """
        pass

    @abc.abstractmethod
    def WithdrawValidatorCommission(self,
        request: cosmos.distribution.v1beta1.tx_pb2.MsgWithdrawValidatorCommission,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.tx_pb2.MsgWithdrawValidatorCommissionResponse:
        """WithdrawValidatorCommission defines a method to withdraw the
        full commission to the validator address.
        """
        pass

    @abc.abstractmethod
    def FundCommunityPool(self,
        request: cosmos.distribution.v1beta1.tx_pb2.MsgFundCommunityPool,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.tx_pb2.MsgFundCommunityPoolResponse:
        """FundCommunityPool defines a method to allow an account to directly
        fund the community pool.
        """
        pass


def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MultiSignature(google.protobuf.message.Message):
    """MultiSignature wraps the signatures from a multisig.LegacyAminoPubKey.
    See cosmos.tx.v1betata1.ModeInfo.Multi for how to specify which signers
    signed and with which modes.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SIGNATURES_FIELD_NUMBER: builtins.int
    @property
    def signatures(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(self,
        *,
        signatures: typing.Optional[typing.Iterable[builtins.bytes]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["signatures",b"signatures"]) -> None: ...
global___MultiSignature = MultiSignature

class CompactBitArray(google.protobuf.message.Message):
    """CompactBitArray is an implementation of a space efficient bit array.
    This is used to ensure that the encoded data takes up a minimal amount of
    space after proto encoding.
    This is not thread safe, and is not intended for concurrent usage.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EXTRA_BITS_STORED_FIELD_NUMBER: builtins.int
    ELEMS_FIELD_NUMBER: builtins.int
    extra_bits_stored: builtins.int
    elems: builtins.bytes
    def __init__(self,
        *,
        extra_bits_stored: builtins.int = ...,
        elems: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["elems",b"elems","extra_bits_stored",b"extra_bits_stored"]) -> None: ...
global___CompactBitArray = CompactBitArray



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Params(google.protobuf.message.Message):
    """Params defines the set of params for the distribution module."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COMMUNITY_TAX_FIELD_NUMBER: builtins.int
    BASE_PROPOSER_REWARD_FIELD_NUMBER: builtins.int
    BONUS_PROPOSER_REWARD_FIELD_NUMBER: builtins.int
    WITHDRAW_ADDR_ENABLED_FIELD_NUMBER: builtins.int
    community_tax: typing.Text
    base_proposer_reward: typing.Text
    bonus_proposer_reward: typing.Text
    withdraw_addr_enabled: builtins.bool
    def __init__(self,
        *,
        community_tax: typing.Text = ...,
        base_proposer_reward: typing.Text = ...,
        bonus_proposer_reward: typing.Text = ...,
        withdraw_addr_enabled: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_proposer_reward",b"base_proposer_reward","bonus_proposer_reward",b"bonus_proposer_reward","community_tax",b"community_tax","withdraw_addr_enabled",b"withdraw_addr_enabled"]) -> None: ...
global___Params = Params

class ValidatorHistoricalRewards(google.protobuf.message.Message):
    """ValidatorHistoricalRewards represents historical rewards for a validator.
    Height is implicit within the store key.
    Cumulative reward ratio is the sum from the zeroeth period
    until this period of rewards / tokens, per the spec.
    The reference count indicates the number of objects
    which might need to reference this historical entry at any point.
    ReferenceCount =
       number of outstanding delegations which ended the associated period (and
       might need to read that record)
     + number of slashes which ended the associated period (and might need to
     read that record)
     + one per validator for the zeroeth period, set on initialization
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CUMULATIVE_REWARD_RATIO_FIELD_NUMBER: builtins.int
    REFERENCE_COUNT_FIELD_NUMBER: builtins.int
    @property
    def cumulative_reward_ratio(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]: ...
    reference_count: builtins.int
    def __init__(self,
        *,
        cumulative_reward_ratio: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin]] = ...,
        reference_count: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["cumulative_reward_ratio",b"cumulative_reward_ratio","reference_count",b"reference_count"]) -> None: ...
global___ValidatorHistoricalRewards = ValidatorHistoricalRewards

class ValidatorCurrentRewards(google.protobuf.message.Message):
    """ValidatorCurrentRewards represents current rewards and current
    period for a validator kept as a running counter and incremented
    each block as long as the validator's tokens remain constant.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REWARDS_FIELD_NUMBER: builtins.int
    PERIOD_FIELD_NUMBER: builtins.int
    @property
    def rewards(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]: ...
    period: builtins.int
    def __init__(self,
        *,
        rewards: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin]] = ...,
        period: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["period",b"period","rewards",b"rewards"]) -> None: ...
global___ValidatorCurrentRewards = ValidatorCurrentRewards

class ValidatorAccumulatedCommission(google.protobuf.message.Message):
    """ValidatorAccumulatedCommission represents accumulated commission
    for a validator kept as a running counter, can be withdrawn at any time.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COMMISSION_FIELD_NUMBER: builtins.int
    @property
    def commission(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]: ...
    def __init__(self,
        *,
        commission: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["commission",b"commission"]) -> None: ...
global___ValidatorAccumulatedCommission = ValidatorAccumulatedCommission

class ValidatorOutstandingRewards(google.protobuf.message.Message):
    """ValidatorOutstandingRewards represents outstanding (un-withdrawn) rewards
    for a validator inexpensive to track, allows simple sanity checks.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    REWARDS_FIELD_NUMBER: builtins.int
    @property
    def rewards(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]: ...
    def __init__(self,
        *,
        rewards: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["rewards",b"rewards"]) -> None: ...
global___ValidatorOutstandingRewards = ValidatorOutstandingRewards

class ValidatorSlashEvent(google.protobuf.message.Message):
    """ValidatorSlashEvent represents a validator slash event.
    Height is implicit within the store key.
    This is needed to calculate appropriate amount of staking tokens
    for delegations which are withdrawn after a slash has occurred.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_PERIOD_FIELD_NUMBER: builtins.int
    FRACTION_FIELD_NUMBER: builtins.int
    validator_period: builtins.int
    fraction: typing.Text
    def __init__(self,
        *,
        validator_period: builtins.int = ...,
        fraction: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["fraction",b"fraction","validator_period",b"validator_period"]) -> None: ...
global___ValidatorSlashEvent = ValidatorSlashEvent

class ValidatorSlashEvents(google.protobuf.message.Message):
    """ValidatorSlashEvents is a collection of ValidatorSlashEvent messages."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_SLASH_EVENTS_FIELD_NUMBER: builtins.int
    @property
    def validator_slash_events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValidatorSlashEvent]: ...
    def __init__(self,
        *,
        validator_slash_events: typing.Optional[typing.Iterable[global___ValidatorSlashEvent]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_slash_events",b"validator_slash_events"]) -> None: ...
global___ValidatorSlashEvents = ValidatorSlashEvents

class FeePool(google.protobuf.message.Message):
    """FeePool is the global fee pool for distribution."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COMMUNITY_POOL_FIELD_NUMBER: builtins.int
    @property
    def community_pool(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]: ...
    def __init__(self,
        *,
        community_pool: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["community_pool",b"community_pool"]) -> None: ...
global___FeePool = FeePool

class CommunityPoolSpendProposal(google.protobuf.message.Message):
    """CommunityPoolSpendProposal details a proposal for use of community funds,
    together with how many coins are proposed to be spent, and to which
    recipient account.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TITLE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    RECIPIENT_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    title: typing.Text
    description: typing.Text
    recipient: typing.Text
    @property
    def amount(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        title: typing.Text = ...,
        description: typing.Text = ...,
        recipient: typing.Text = ...,
        amount: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","description",b"description","recipient",b"recipient","title",b"title"]) -> None: ...
global___CommunityPoolSpendProposal = CommunityPoolSpendProposal

class DelegatorStartingInfo(google.protobuf.message.Message):
    """DelegatorStartingInfo represents the starting info for a delegator reward
    period. It tracks the previous validator period, the delegation's amount of
    staking token, and the creation height (to check later on if any slashes have
    occurred). NOTE: Even though validators are slashed to whole staking tokens,
    the delegators within the validator may be left with less than a full token,
    thus sdk.Dec is used.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PREVIOUS_PERIOD_FIELD_NUMBER: builtins.int
    STAKE_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    previous_period: builtins.int
    stake: typing.Text
    height: builtins.int
    def __init__(self,
        *,
        previous_period: builtins.int = ...,
        stake: typing.Text = ...,
        height: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["height",b"height","previous_period",b"previous_period","stake",b"stake"]) -> None: ...
global___DelegatorStartingInfo = DelegatorStartingInfo

class DelegationDelegatorReward(google.protobuf.message.Message):
    """DelegationDelegatorReward represents the properties
    of a delegator's delegation reward.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDRESS_FIELD_NUMBER: builtins.int
    REWARD_FIELD_NUMBER: builtins.int
    validator_address: typing.Text
    @property
    def reward(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.DecCoin]: ...
    def __init__(self,
        *,
        validator_address: typing.Text = ...,
        reward: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.DecCoin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["reward",b"reward","validator_address",b"validator_address"]) -> None: ...
global___DelegationDelegatorReward = DelegationDelegatorReward

class CommunityPoolSpendProposalWithDeposit(google.protobuf.message.Message):
    """CommunityPoolSpendProposalWithDeposit defines a CommunityPoolSpendProposal
    with a deposit
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TITLE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    RECIPIENT_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    DEPOSIT_FIELD_NUMBER: builtins.int
    title: typing.Text
    description: typing.Text
    recipient: typing.Text
    amount: typing.Text
    deposit: typing.Text
    def __init__(self,
        *,
        title: typing.Text = ...,
        description: typing.Text = ...,
        recipient: typing.Text = ...,
        amount: typing.Text = ...,
        deposit: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","deposit",b"deposit","description",b"description","recipient",b"recipient","title",b"title"]) -> None: ...
global___CommunityPoolSpendProposalWithDeposit = CommunityPoolSpendProposalWithDeposit



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class EpochInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    IDENTIFIER_FIELD_NUMBER: builtins.int
    START_TIME_FIELD_NUMBER: builtins.int
    DURATION_FIELD_NUMBER: builtins.int
    CURRENT_EPOCH_FIELD_NUMBER: builtins.int
    CURRENT_EPOCH_START_TIME_FIELD_NUMBER: builtins.int
    EPOCH_COUNTING_STARTED_FIELD_NUMBER: builtins.int
    CURRENT_EPOCH_START_HEIGHT_FIELD_NUMBER: builtins.int
    identifier: typing.Text
    """A string identifier for the epoch. e.g. "15min" or "1hour" """

    @property
    def start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """When the epoch repetitino should start."""
        pass
    @property
    def duration(self) -> google.protobuf.duration_pb2.Duration:
        """How long each epoch lasts for."""
        pass
    current_epoch: builtins.int
    """The current epoch number, starting from 1."""

    @property
    def current_epoch_start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """The start timestamp of the current epoch."""
        pass
    epoch_counting_started: builtins.bool
    """Whether or not this epoch has started. Set to true if current blocktime >= start_time."""

    current_epoch_start_height: builtins.int
    """The block height at which the current epoch started at."""

    def __init__(self,
        *,
        identifier: typing.Text = ...,
        start_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        duration: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        current_epoch: builtins.int = ...,
        current_epoch_start_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        epoch_counting_started: builtins.bool = ...,
        current_epoch_start_height: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["current_epoch_start_time",b"current_epoch_start_time","duration",b"duration","start_time",b"start_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["current_epoch",b"current_epoch","current_epoch_start_height",b"current_epoch_start_height","current_epoch_start_time",b"current_epoch_start_time","duration",b"duration","epoch_counting_started",b"epoch_counting_started","identifier",b"identifier","start_time",b"start_time"]) -> None: ...
global___EpochInfo = EpochInfo

class GenesisState(google.protobuf.message.Message):
    """GenesisState defines the epochs module's genesis state."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EPOCHS_FIELD_NUMBER: builtins.int
    @property
    def epochs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___EpochInfo]: ...
    def __init__(self,
        *,
        epochs: typing.Optional[typing.Iterable[global___EpochInfo]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["epochs",b"epochs"]) -> None: ...
global___GenesisState = GenesisState



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgMintStable(google.protobuf.message.Message):
    """
    MsgMintStable: Msg to mint NUSD. A user deposits NIBI and collateral and gets 
    NUSD in return. The amount of NUSD received depends on the current price set 
    by the pricefeed library and the current collateral ratio for the protocol.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CREATOR_FIELD_NUMBER: builtins.int
    STABLE_FIELD_NUMBER: builtins.int
    creator: typing.Text
    @property
    def stable(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        creator: typing.Text = ...,
        stable: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["stable",b"stable"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["creator",b"creator","stable",b"stable"]) -> None: ...
global___MsgMintStable = MsgMintStable

class MsgMintStableResponse(google.protobuf.message.Message):
    """MsgMintStableResponse specifies the amount of NUSD token the user will receive after their
    mint transaction
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    STABLE_FIELD_NUMBER: builtins.int
    USED_COINS_FIELD_NUMBER: builtins.int
    FEES_PAYED_FIELD_NUMBER: builtins.int
    @property
    def stable(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    @property
    def used_coins(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def fees_payed(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        stable: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        used_coins: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        fees_payed: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["stable",b"stable"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["fees_payed",b"fees_payed","stable",b"stable","used_coins",b"used_coins"]) -> None: ...
global___MsgMintStableResponse = MsgMintStableResponse

class MsgBurnStable(google.protobuf.message.Message):
    """
    MsgBurnStable allows users to burn NUSD in exchange for NIBI and collateral. 
    The amount of NIBI and Collateral received depends on the current price set by 
    the x/pricefeed library and the current collateral ratio.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CREATOR_FIELD_NUMBER: builtins.int
    STABLE_FIELD_NUMBER: builtins.int
    creator: typing.Text
    @property
    def stable(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        creator: typing.Text = ...,
        stable: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["stable",b"stable"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["creator",b"creator","stable",b"stable"]) -> None: ...
global___MsgBurnStable = MsgBurnStable

class MsgBurnStableResponse(google.protobuf.message.Message):
    """MsgBurnStableResponse specifies the amount of collateral and governance 
    token the user will receive after their burn transaction.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COLLATERAL_FIELD_NUMBER: builtins.int
    GOV_FIELD_NUMBER: builtins.int
    FEES_PAYED_FIELD_NUMBER: builtins.int
    @property
    def collateral(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    @property
    def gov(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    @property
    def fees_payed(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        collateral: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        gov: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        fees_payed: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["collateral",b"collateral","gov",b"gov"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["collateral",b"collateral","fees_payed",b"fees_payed","gov",b"gov"]) -> None: ...
global___MsgBurnStableResponse = MsgBurnStableResponse

class MsgRecollateralize(google.protobuf.message.Message):
    """MsgRecollateralize"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CREATOR_FIELD_NUMBER: builtins.int
    COLL_FIELD_NUMBER: builtins.int
    creator: typing.Text
    @property
    def coll(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        creator: typing.Text = ...,
        coll: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["coll",b"coll"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coll",b"coll","creator",b"creator"]) -> None: ...
global___MsgRecollateralize = MsgRecollateralize

class MsgRecollateralizeResponse(google.protobuf.message.Message):
    """MsgRecollateralizeResponse is the output of a successful 'Recollateralize'"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GOV_FIELD_NUMBER: builtins.int
    @property
    def gov(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """Gov (sdk.Coin): Tokens rewarded to the caller in exchange for her collateral."""
        pass
    def __init__(self,
        *,
        gov: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["gov",b"gov"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["gov",b"gov"]) -> None: ...
global___MsgRecollateralizeResponse = MsgRecollateralizeResponse

class MsgBuyback(google.protobuf.message.Message):
    """MsgBuyback"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CREATOR_FIELD_NUMBER: builtins.int
    GOV_FIELD_NUMBER: builtins.int
    creator: typing.Text
    @property
    def gov(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """Gov (sdk.Coin): Tokens the caller wants to sell to the protocol in exchange 
        for collateral.
        """
        pass
    def __init__(self,
        *,
        creator: typing.Text = ...,
        gov: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["gov",b"gov"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["creator",b"creator","gov",b"gov"]) -> None: ...
global___MsgBuyback = MsgBuyback

class MsgBuybackResponse(google.protobuf.message.Message):
    """MsgBuybackResponse is the output of a successful 'Buyback'"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COLL_FIELD_NUMBER: builtins.int
    @property
    def coll(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """Coll (sdk.Coin): Tokens sold to the caller in exchange for her collateral."""
        pass
    def __init__(self,
        *,
        coll: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["coll",b"coll"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["coll",b"coll"]) -> None: ...
global___MsgBuybackResponse = MsgBuybackResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import dex.v1.query_pb2
import grpc

class QueryStub:
    """Query defines the gRPC querier service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    Params: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryParamsRequest,
        dex.v1.query_pb2.QueryParamsResponse]
    """Parameters of the dex module."""

    PoolNumber: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryPoolNumberRequest,
        dex.v1.query_pb2.QueryPoolNumberResponse]
    """Next available pool id number."""

    Pool: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryPoolRequest,
        dex.v1.query_pb2.QueryPoolResponse]
    """Fetch a pool by id."""

    Pools: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryPoolsRequest,
        dex.v1.query_pb2.QueryPoolsResponse]
    """Returns all pools."""

    PoolParams: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryPoolParamsRequest,
        dex.v1.query_pb2.QueryPoolParamsResponse]
    """Parameters of a single pool."""

    NumPools: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryNumPoolsRequest,
        dex.v1.query_pb2.QueryNumPoolsResponse]
    """Number of pools."""

    TotalLiquidity: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryTotalLiquidityRequest,
        dex.v1.query_pb2.QueryTotalLiquidityResponse]
    """Total liquidity across all pools."""

    TotalPoolLiquidity: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryTotalPoolLiquidityRequest,
        dex.v1.query_pb2.QueryTotalPoolLiquidityResponse]
    """Total liquidity in a single pool."""

    TotalShares: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryTotalSharesRequest,
        dex.v1.query_pb2.QueryTotalSharesResponse]
    """Total shares in a single pool."""

    SpotPrice: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QuerySpotPriceRequest,
        dex.v1.query_pb2.QuerySpotPriceResponse]
    """Instantaneous price of an asset in a pool."""

    EstimateSwapExactAmountIn: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QuerySwapExactAmountInRequest,
        dex.v1.query_pb2.QuerySwapExactAmountInResponse]
    """Estimates the amount of assets returned given an exact amount of tokens to
    swap.
    """

    EstimateSwapExactAmountOut: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QuerySwapExactAmountOutRequest,
        dex.v1.query_pb2.QuerySwapExactAmountOutResponse]
    """Estimates the amount of tokens required to return the exact amount of
    assets requested.
    """

    EstimateJoinExactAmountIn: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryJoinExactAmountInRequest,
        dex.v1.query_pb2.QueryJoinExactAmountInResponse]
    """Estimates the amount of pool shares returned given an amount of tokens to
    join.
    """

    EstimateJoinExactAmountOut: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryJoinExactAmountOutRequest,
        dex.v1.query_pb2.QueryJoinExactAmountOutResponse]
    """Estimates the amount of tokens required to obtain an exact amount of pool
    shares.
    """

    EstimateExitExactAmountIn: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryExitExactAmountInRequest,
        dex.v1.query_pb2.QueryExitExactAmountInResponse]
    """Estimates the amount of tokens returned to the user given an exact amount
    of pool shares.
    """

    EstimateExitExactAmountOut: grpc.UnaryUnaryMultiCallable[
        dex.v1.query_pb2.QueryExitExactAmountOutRequest,
        dex.v1.query_pb2.QueryExitExactAmountOutResponse]
    """Estimates the amount of pool shares required to extract an exact amount of
    tokens from the pool.
    """


class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service."""
    @abc.abstractmethod
    def Params(self,
        request: dex.v1.query_pb2.QueryParamsRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryParamsResponse:
        """Parameters of the dex module."""
        pass

    @abc.abstractmethod
    def PoolNumber(self,
        request: dex.v1.query_pb2.QueryPoolNumberRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryPoolNumberResponse:
        """Next available pool id number."""
        pass

    @abc.abstractmethod
    def Pool(self,
        request: dex.v1.query_pb2.QueryPoolRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryPoolResponse:
        """Fetch a pool by id."""
        pass

    @abc.abstractmethod
    def Pools(self,
        request: dex.v1.query_pb2.QueryPoolsRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryPoolsResponse:
        """Returns all pools."""
        pass

    @abc.abstractmethod
    def PoolParams(self,
        request: dex.v1.query_pb2.QueryPoolParamsRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryPoolParamsResponse:
        """Parameters of a single pool."""
        pass

    @abc.abstractmethod
    def NumPools(self,
        request: dex.v1.query_pb2.QueryNumPoolsRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryNumPoolsResponse:
        """Number of pools."""
        pass

    @abc.abstractmethod
    def TotalLiquidity(self,
        request: dex.v1.query_pb2.QueryTotalLiquidityRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryTotalLiquidityResponse:
        """Total liquidity across all pools."""
        pass

    @abc.abstractmethod
    def TotalPoolLiquidity(self,
        request: dex.v1.query_pb2.QueryTotalPoolLiquidityRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryTotalPoolLiquidityResponse:
        """Total liquidity in a single pool."""
        pass

    @abc.abstractmethod
    def TotalShares(self,
        request: dex.v1.query_pb2.QueryTotalSharesRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryTotalSharesResponse:
        """Total shares in a single pool."""
        pass

    @abc.abstractmethod
    def SpotPrice(self,
        request: dex.v1.query_pb2.QuerySpotPriceRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QuerySpotPriceResponse:
        """Instantaneous price of an asset in a pool."""
        pass

    @abc.abstractmethod
    def EstimateSwapExactAmountIn(self,
        request: dex.v1.query_pb2.QuerySwapExactAmountInRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QuerySwapExactAmountInResponse:
        """Estimates the amount of assets returned given an exact amount of tokens to
        swap.
        """
        pass

    @abc.abstractmethod
    def EstimateSwapExactAmountOut(self,
        request: dex.v1.query_pb2.QuerySwapExactAmountOutRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QuerySwapExactAmountOutResponse:
        """Estimates the amount of tokens required to return the exact amount of
        assets requested.
        """
        pass

    @abc.abstractmethod
    def EstimateJoinExactAmountIn(self,
        request: dex.v1.query_pb2.QueryJoinExactAmountInRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryJoinExactAmountInResponse:
        """Estimates the amount of pool shares returned given an amount of tokens to
        join.
        """
        pass

    @abc.abstractmethod
    def EstimateJoinExactAmountOut(self,
        request: dex.v1.query_pb2.QueryJoinExactAmountOutRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryJoinExactAmountOutResponse:
        """Estimates the amount of tokens required to obtain an exact amount of pool
        shares.
        """
        pass

    @abc.abstractmethod
    def EstimateExitExactAmountIn(self,
        request: dex.v1.query_pb2.QueryExitExactAmountInRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryExitExactAmountInResponse:
        """Estimates the amount of tokens returned to the user given an exact amount
        of pool shares.
        """
        pass

    @abc.abstractmethod
    def EstimateExitExactAmountOut(self,
        request: dex.v1.query_pb2.QueryExitExactAmountOutRequest,
        context: grpc.ServicerContext,
    ) -> dex.v1.query_pb2.QueryExitExactAmountOutResponse:
        """Estimates the amount of pool shares required to extract an exact amount of
        tokens from the pool.
        """
        pass


def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



import json
import logging
from copy import deepcopy

from google.protobuf import message
from google.protobuf.json_format import MessageToDict

from nibiru.client import Client
from nibiru.common import TxConfig, TxType
from nibiru.composer import Composer
from nibiru.constant import GAS_PRICE
from nibiru.exceptions import SimulationError, TxError
from nibiru.network import Network
from nibiru.proto.cosmos.base.abci.v1beta1 import abci_pb2 as abci_type
from nibiru.transaction import Transaction
from nibiru.wallet import PrivateKey


class Tx:
    def __init__(self, priv_key: PrivateKey, network: Network, client: Client, config: TxConfig):
        self.priv_key = priv_key
        self.network = network
        self.client = client
        self.address = None
        self.config = config
        self.msgs = []

    def add_messages(self, *msgs: message.Message):
        self.msgs.extend(msgs)
        return self

    def execute(self, **kwargs):
        try:
            res = self.execute_msg(*self.msgs, **kwargs)
        except SimulationError as err:
            raise err
        except TxError as err:
            raise err
        else:
            self.msgs = []

        return res

    def execute_msg(
        self, *msg: message.Message, get_sequence_from_node: bool = False, **kwargs
    ) -> abci_type.TxResponse:
        """
        Execute a message to broadcast a transaction to the node.
        Simulate the message to generate the gas estimate and send it to the node.
        If the transaction fail because of account sequence mismatch, we try to send it again once more with the
        sequence coming from a query to the lcd endpoint.

        Args:
            get_sequence_from_node (bool, optional): Wether to get the sequence from the local value or the lcd
                endpoint. Defaults to False.

        Raises:
            TxError: Will log the error to the

        Returns:
            abci_type.TxResponse: _description_
        """
        address = None
        sequence_args = {}
        if get_sequence_from_node:
            sequence_args = {"from_node": True, "lcd_endpoint": self.network.lcd_endpoint}

        try:
            self.client.sync_timeout_height()
            address = self.get_address_info()
            tx = (
                Transaction()
                .with_messages(*msg)
                .with_sequence(address.get_sequence(**sequence_args))
                .with_account_num(address.get_number())
                .with_chain_id(self.network.chain_id)
                .with_signer(self.priv_key)
            )
            sim_res = self.simulate(tx)
            gas_estimate = sim_res.gas_info.gas_used
            tx_output = self.execute_tx(tx, gas_estimate, **kwargs)

            if tx_output.code != 0:
                address.decrease_sequence()
                raise TxError(tx_output.raw_log)

            tx_output = MessageToDict(tx_output)

            # Convert raw log into a dictionary
            tx_output["rawLog"] = json.loads(tx_output.get("rawLog", "{}"))
            return tx_output

        except SimulationError as err:
            if "account sequence mismatch, expected" in str(err) and not get_sequence_from_node:
                return self.execute_msg(*msg, get_sequence_from_node=True, **kwargs)

            if address:
                address.decrease_sequence()

            raise TxError(f"Failed to execute transaction: {err}") from err

    def execute_tx(self, tx: Transaction, gas_estimate: float, **kwargs):
        conf = self.get_config(**kwargs)
        gas_wanted = gas_estimate * 1.25
        if conf.gas_wanted > 0:
            gas_wanted = conf.gas_wanted
        elif conf.gas_multiplier > 0:
            gas_wanted = gas_estimate * conf.gas_multiplier
        gas_price = GAS_PRICE if conf.gas_price <= 0 else conf.gas_price

        fee = [
            Composer.coin(
                amount=int(gas_price * gas_wanted),
                denom=self.network.fee_denom,
            )
        ]
        logging.info("Executing transaction with fee: %s and gas_wanted: %d", fee, gas_wanted)
        tx = tx.with_gas(gas_wanted).with_fee(fee).with_memo("").with_timeout_height(self.client.timeout_height)
        tx_raw_bytes = tx.get_signed_tx_data()

        return self._send_tx(tx_raw_bytes, conf.tx_type)

    def _send_tx(self, tx_raw_bytes, tx_type: TxType):
        if tx_type == TxType.SYNC:
            return self.client.send_tx_sync_mode(tx_raw_bytes)
        elif tx_type == TxType.ASYNC:
            return self.client.send_tx_async_mode(tx_raw_bytes)

        return self.client.send_tx_block_mode(tx_raw_bytes)

    def simulate(self, tx: Transaction):
        sim_tx_raw_bytes = tx.get_signed_tx_data()

        (sim_res, success) = self.client.simulate_tx(sim_tx_raw_bytes)
        if not success:
            raise SimulationError(sim_res)

        return sim_res

    def get_address_info(self):
        if self.address is None:
            pub_key = self.priv_key.to_public_key()
            self.address = pub_key.to_address()
            self.address.init_num_seq(self.network.lcd_endpoint)

        return self.address

    def get_config(self, **kwargs):
        """
        Properties in kwargs overwrite config
        """
        c = deepcopy(self.config)
        props = dir(c)
        for (k, v) in kwargs.items():
            if k in props:
                setattr(c, k, v)
            else:
                logging.warning("%s is not a supported config property, ignoring", k)

        return c



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import vpool.v1.query_pb2

class QueryStub:
    """Query defines the gRPC querier service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    ReserveAssets: grpc.UnaryUnaryMultiCallable[
        vpool.v1.query_pb2.QueryReserveAssetsRequest,
        vpool.v1.query_pb2.QueryReserveAssetsResponse]
    """Queries the reserve assets in a given pool, identified by a token pair."""

    AllPools: grpc.UnaryUnaryMultiCallable[
        vpool.v1.query_pb2.QueryAllPoolsRequest,
        vpool.v1.query_pb2.QueryAllPoolsResponse]
    """Queries all virtual pools."""

    BaseAssetPrice: grpc.UnaryUnaryMultiCallable[
        vpool.v1.query_pb2.QueryBaseAssetPriceRequest,
        vpool.v1.query_pb2.QueryBaseAssetPriceResponse]
    """Queries prices"""


class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service."""
    @abc.abstractmethod
    def ReserveAssets(self,
        request: vpool.v1.query_pb2.QueryReserveAssetsRequest,
        context: grpc.ServicerContext,
    ) -> vpool.v1.query_pb2.QueryReserveAssetsResponse:
        """Queries the reserve assets in a given pool, identified by a token pair."""
        pass

    @abc.abstractmethod
    def AllPools(self,
        request: vpool.v1.query_pb2.QueryAllPoolsRequest,
        context: grpc.ServicerContext,
    ) -> vpool.v1.query_pb2.QueryAllPoolsResponse:
        """Queries all virtual pools."""
        pass

    @abc.abstractmethod
    def BaseAssetPrice(self,
        request: vpool.v1.query_pb2.QueryBaseAssetPriceRequest,
        context: grpc.ServicerContext,
    ) -> vpool.v1.query_pb2.QueryBaseAssetPriceResponse:
        """Queries prices"""
        pass


def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.authz.v1beta1.authz_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgGrant(google.protobuf.message.Message):
    """MsgGrant is a request type for Grant method. It declares authorization to the grantee
    on behalf of the granter with the provided expiration time.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GRANTER_FIELD_NUMBER: builtins.int
    GRANTEE_FIELD_NUMBER: builtins.int
    GRANT_FIELD_NUMBER: builtins.int
    granter: typing.Text
    grantee: typing.Text
    @property
    def grant(self) -> cosmos.authz.v1beta1.authz_pb2.Grant: ...
    def __init__(self,
        *,
        granter: typing.Text = ...,
        grantee: typing.Text = ...,
        grant: typing.Optional[cosmos.authz.v1beta1.authz_pb2.Grant] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["grant",b"grant"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["grant",b"grant","grantee",b"grantee","granter",b"granter"]) -> None: ...
global___MsgGrant = MsgGrant

class MsgExecResponse(google.protobuf.message.Message):
    """MsgExecResponse defines the Msg/MsgExecResponse response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    RESULTS_FIELD_NUMBER: builtins.int
    @property
    def results(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(self,
        *,
        results: typing.Optional[typing.Iterable[builtins.bytes]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["results",b"results"]) -> None: ...
global___MsgExecResponse = MsgExecResponse

class MsgExec(google.protobuf.message.Message):
    """MsgExec attempts to execute the provided messages using
    authorizations granted to the grantee. Each message should have only
    one signer corresponding to the granter of the authorization.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GRANTEE_FIELD_NUMBER: builtins.int
    MSGS_FIELD_NUMBER: builtins.int
    grantee: typing.Text
    @property
    def msgs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]:
        """Authorization Msg requests to execute. Each msg must implement Authorization interface
        The x/authz will try to find a grant matching (msg.signers[0], grantee, MsgTypeURL(msg))
        triple and validate it.
        """
        pass
    def __init__(self,
        *,
        grantee: typing.Text = ...,
        msgs: typing.Optional[typing.Iterable[google.protobuf.any_pb2.Any]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["grantee",b"grantee","msgs",b"msgs"]) -> None: ...
global___MsgExec = MsgExec

class MsgGrantResponse(google.protobuf.message.Message):
    """MsgGrantResponse defines the Msg/MsgGrant response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgGrantResponse = MsgGrantResponse

class MsgRevoke(google.protobuf.message.Message):
    """MsgRevoke revokes any authorization with the provided sdk.Msg type on the
    granter's account with that has been granted to the grantee.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GRANTER_FIELD_NUMBER: builtins.int
    GRANTEE_FIELD_NUMBER: builtins.int
    MSG_TYPE_URL_FIELD_NUMBER: builtins.int
    granter: typing.Text
    grantee: typing.Text
    msg_type_url: typing.Text
    def __init__(self,
        *,
        granter: typing.Text = ...,
        grantee: typing.Text = ...,
        msg_type_url: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["grantee",b"grantee","granter",b"granter","msg_type_url",b"msg_type_url"]) -> None: ...
global___MsgRevoke = MsgRevoke

class MsgRevokeResponse(google.protobuf.message.Message):
    """MsgRevokeResponse defines the Msg/MsgRevokeResponse response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgRevokeResponse = MsgRevokeResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class PriceSnapshot(google.protobuf.message.Message):
    """a snapshot of the pricefeed oracle's median price at a given point in time"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAIR_ID_FIELD_NUMBER: builtins.int
    PRICE_FIELD_NUMBER: builtins.int
    TIMESTAMP_MS_FIELD_NUMBER: builtins.int
    pair_id: typing.Text
    """the token pair"""

    price: typing.Text
    """the median prices of all oracles"""

    timestamp_ms: builtins.int
    """milliseconds since unix epoch"""

    def __init__(self,
        *,
        pair_id: typing.Text = ...,
        price: typing.Text = ...,
        timestamp_ms: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pair_id",b"pair_id","price",b"price","timestamp_ms",b"timestamp_ms"]) -> None: ...
global___PriceSnapshot = PriceSnapshot
